"""
This package contains wrappers for other topic modeling programs.
"""

from .ldamallet import LdaMallet
from .dtmmodel import DtmModel
from .ldavowpalwabbit import LdaVowpalWabbit
from .fasttext import FastText
from .wordrank import Wordrank
from .varembed import VarEmbed